#! /usr/bin/env python

from lightflowPM import *

s = scene()

s.lightOn( s.newLight( "point", [ "position", vector3( 5.0, -5.0, 4.0 ), "color", vector3( 300.0, 300.0, 300.0 ) ] ) )

s.transformBegin( transform().rotationAroundX( 3.141592 / 2 ) )

bump = s.newPattern( "angular",
		     [ "depth",
		       0.0, 0.0, 0.0,
		       0.5, 1.0, 1.0,
		       1.0, 0.0, 0.0,
		       "depth", 0.2,
		       "scale", 0.1 ] )

s.transformEnd()

blotchy = s.newPattern( "multifractal",
			[ "color",
			  0.0, vector3(0.3, 0.3, 0.3), vector3(0.3, 0.3, 0.3),
  			  1.0, vector3(1.0, 1.0, 1.0), vector3(1.0, 1.0, 1.0),
			  "basis", "sin",
			  "scale", 10.0,
			  "turbulence.omega", 0.3, 0.5,
			  "turbulence.octaves", 3] )

envmap  = s.newPattern( "envmap",
			[ "pattern", blotchy,
			  "center", vector3( 0, 0, 0 ),
			  "radius", 10.0,
			  "reflection", vector3( 0.9, 0.9, 1 ),
			  "refraction", vector3( 0.0, 0.0, 0 ),
    			  "IOR", 8.0,
			  "fresnel", 1 ] )

metal = s.newMaterial( "generic",
		       [ "fresnel", 1,
			 "IOR", 8.0,
			 "ka", vector3( 1, 1, 1 ),
			 "ka", envmap,
			 "ksr", vector3( 0.9, 0.9, 1 ), vector3( 0.9, 0.6, 1.0 ),
			 "km", 0.05,
			 "shinyness", 0.95,
			 "displacement", bump,
			 "radiosity", 0 ] )


s.materialBegin( metal )

sphere = s.newObject( "sphere", [ "radius", 1.0 ] )

s.addObject( s.newObject( "surface-engine",
			  [ "surfaces", sphere,
			    "tolerance", 0.02, 0.1, 0.05 ] ) )

s.materialEnd()


saver = s.newImager( "tga-saver", [ "file", "angular.tga" ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole", [ "eye", vector3( 0, -4, 0 ), "aim", vector3( 0, 0, 0 ) ] )

s.imagerEnd()

s.render( camera, 300, 300 )
