#! /usr/bin/env python

from lightflowPM import *
from math import *

s = scene()

s.lightOn( s.newLight( "point", [ "position", vector3( 4.0, -5.0, 4.0 ), "color", vector3( 250.0, 250.0, 250.0 ) ] ) )
s.lightOn( s.newLight( "point", [ "position", vector3( -6.0, -3.0, -4.0 ), "color", vector3( 80.0, 0.0, 0.0 ) ] ) )


# make a black reference pattern that will be used to make masking
black = s.newPattern( "tint",
                      [ "color", vector3( 0, 0, 0 ) ] )

# make a map to use as a mask for another pattern
map = s.newPattern( "map",
                    [ "texture", s.newTexture( "../Textures/hnytxt.tga" ),
                      "scale", 0.5 ] )

# make a pattern
wood = s.newPattern( "wood",
		     [ "color",
		       1.0, vector3( 1.0, 0.85, 0.7 ), vector3( 1.0, 0.85, 0.7 ),
		       0.0, vector3( 0.8, 0.60, 0.4 ), vector3( 0.8, 0.60, 0.4 ),
		       "scale", 0.15,
		       "turbulence.amount", 0.5,
		       "turbulence.frequency", 2.0,
		       "turbulence.octaves", 1 ] )

# create the masked pattern, using a blend pattern that associates dark
# levels of the mask with black, and bright levels of the mask with wood
layer1 = s.newPattern( "blend",
                       [ "pattern", map, 
                         "gradient",
                         0.0, black, black,
                         1.0, wood, wood ] )

s.transformBegin( transform().rotationAroundZ( pi / 7 ) )
s.transformBegin( transform().rotationAroundX( pi / 6 ) )

marble1 = s.newPattern( "agate",
			[ "color",
			  0.00, vector3( 0.4, 0.40, 0.4 ), vector3( 0.4, 0.40, 0.4 ),
			  0.50, vector3( 1.0, 1.00, 1.0 ), vector3( 1.0, 1.00, 1.0 ),
			  "scale", 0.4 ] )

marble2 = s.newPattern( "multifractal",
			[ "color",
			  0.00, vector3( 0.8, 0.6, 0.5 ), vector3( 0.8, 0.6, 0.5 ),
			  0.50, vector3( 1.0, 1.0, 1.0 ), vector3( 1.0, 1.0, 1.0 ),
			  "scale", 0.4 ] )

layer2 = s.newPattern( "compose",
		       [ "patterns", marble2, marble1 ] )

s.transformEnd()
s.transformEnd()

# mix layers together
mix = s.newPattern( "mix",
                    [ "mix", layer1, 0.6,
                      "mix", layer2, 0.4 ] )

plastic = s.newMaterial( "standard",
			 [ "kc", vector3( 0.8, 0.8, 0.8 ),
			   "kd", 0.7,
			   "km", 0.1,
			   "kc", mix ] )

s.materialBegin( plastic )

s.addObject( s.newObject( "sphere",
			  [ "radius", 1.0,
			    "texture",
			    vector2(0.0, 1.0), vector2(1.0, 1.0),
			    vector2(0.0, 0.0), vector2(1.0, 0.0) ] ) )

s.materialEnd()


saver = s.newImager( "tga-saver", [ "file", "layers.tga" ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole", [ "eye", vector3( -1, -4, 0 ), "aim", vector3( 0, 0, 0 ) ] )

s.imagerEnd()

s.render( camera, 300, 300 )
