#! /usr/bin/env python

from lightflowPM import *

s = scene()

s.newInterface( "default",
		[ "trace-depth", 6,
		  "radiosity-depth", 6, 6,
		  "radiosity-sampling", "default",
		  "radiosity-samples", 350,
		  "radiosity-threshold", 0.1,
		  "radiosity-reuse-distance", 0.8, 0.01,
		  "photon-count", 300000,
		  "photon-clustering-count", 1000, 75 ] )

EDGE = 10.0

for i in range(0, 16) :
    for j in range(0, 16) :
	s.transformBegin( transform().translation( vector3( ((i/15.0)*2.0 - 1.0)*(EDGE-0.1), ((j/15.0)*2.0 - 1.0)*(EDGE-0.1), 3.99 ) ) )

	s.lightOn( s.newLight( "conic",
			       [ "position", vector3( 0.0, 0.0, 0.0 ),
				 "direction", vector3( 0.0, 0.0, -1.0 ),
				 "angle", 3.14 / 10.0, 3.14 / 8.0,
				 "color", vector3( 8.0, 8.0, 8.0 ) ] ) )

	s.transformEnd()

neon = s.newMaterial( "matte", [ "kc", vector3( 3, 3, 3 ), "shadowing", 0.0 ] )

whitewash = s.newMaterial( "generic",
                           [ "kdr", vector3( 0.8, 0.8, 0.8 ),
                             "ksr", vector3( 0.5, 0.5, 0.5 ),
                             "km", 0.07,
                             "shinyness", 1.0,
                             "radiosity", 1,
                             "caustics", 0, 0 ] )

check = s.newPattern( "check",
		      [ "color", vector3( 0.8, 0.3, 0.3 ),
			"scale", 1.0 / 30.0 ] )

redwash = s.newMaterial( "generic",
                         [ "kdr", vector3( 0.8, 0.7, 0.7 ),
			   "kdr", check,
                           "ksr", vector3( 0.5, 0.5, 0.5 ),
                           "km", 0.07,
			   "kr", vector3( 0.1, 0.1, 0.1 ),
                           "shinyness", 1.0,
                           "radiosity", 1,
                           "caustics", 0, 0] )

plastic = s.newMaterial( "generic",
			 [ "kdr", vector3( 0.7, 0.7, 1.0 ),
			   "ksr", vector3( 1.0, 1.0, 1.0 ), vector3( 0.0, 1.0, 0.5 ),
			   "km", 0.02 ] )


s.materialBegin( neon )

for i in range(0, 16) :
    for j in range(0, 16) :
	s.transformBegin( transform().translation( vector3( ((i/15.0)*2.0 - 1.0)*(EDGE-0.1), ((j/15.0)*2.0 - 1.0)*(EDGE-0.1), 3.99 ) ) )

	s.addObject( s.newObject( "sphere", [ "radius", 0.05 ] ) )

	s.transformEnd()

s.materialEnd()

s.materialBegin( redwash )

s.addObject( s.newObject( "patch", [ "points",
				     vector3( -EDGE, -EDGE, 0.0 ), vector3( EDGE, -EDGE, 0.0 ),
				     vector3( -EDGE,  EDGE, 0.0 ), vector3( EDGE,  EDGE, 0.0 ) ] ) )

s.materialEnd()

s.materialBegin( whitewash )

s.addObject( s.newObject( "box", [ "position",
				     vector3( -EDGE, -EDGE, -0.1 ), vector3( EDGE, EDGE, 4.0 ) ] ) )

s.materialEnd()


s.materialBegin( plastic )

for i in range(0, 16) :
    for j in range(0, 16) :
	s.transformBegin( transform().translation( vector3( ((i/15.0)*2.0 - 1.0)*(EDGE-1.0), ((j/15.0)*2.0 - 1.0)*(EDGE-1.0), 0.3 ) ) )

	s.addObject( s.newObject( "sphere", [ "radius", 0.15 ] ) )

	s.transformEnd()

s.materialEnd()


saver = s.newImager( "tga-saver", [ "file", "lights256rad.tga" ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole", [ "eye", vector3( 0, -9, 1.5 ), "aim", vector3( 0, 0, 1.5 ), "fov", 3.141592 / 1.8 ] )

s.imagerEnd()


s.radiosity()

s.render( camera, 300, 300 )
