#! /usr/bin/env python

from lightflowPM import *
from math import *
import sys

s = scene()

s.lightOn( s.newLight( "soft", [ "position", vector3( 5.0, -5.0, 4.0 ), "color", vector3( 300.0, 300.0, 300.0 ) ] ) )


plastic = s.newMaterial( "standard", [ "ka", vector3( 0.2, 0.3, 0.4 ), "kc", vector3( 0.5, 0.8, 1.0 ), "kd", 0.5, "km", 0.1 ] )


s.materialBegin( plastic )

p = vector3array()
n = vector3array()
t = vector2array()
nv = intarray()
fp = intarray()
fn = intarray()
ft = intarray()

N = 200
M = 40
R = 1.0
r = 0.2

#alloc arrays
p.setSize( N * M )
n.setSize( N * M )
t.setSize( N * M )
nv.setSize( (N-1) * (M-1) )
fp.setSize( (N-1) * (M-1) * 4 )
fn.setSize( (N-1) * (M-1) * 4 )
ft.setSize( (N-1) * (M-1) * 4 )

for i in range( 0, N ) :
    u = 2.0 * pi * i / (N-1)

    for j in range( 0, M ) :
	v = 2.0 * pi * j / (M-1)

        index = j + i*M

	p.set( index, vector3( cos(u) * (R + cos(v)*r + sin(u*8)*r),
			       sin(u) * (R + cos(v)*r + sin(u*8)*r),
			       sin(v)*r + cos(u*8)*r ) )

	du = vector3( -sin(u) * (R + cos(v)*r + sin(u*8)*r) + cos(u) * cos(u*8)*r*8,
		       cos(u) * (R + cos(v)*r + sin(u*8)*r) + sin(u) * cos(u*8)*r*8,
		      -sin(u*8)*r*8 )

	dv = vector3( -cos(u) * sin(v)*r,
		      -sin(u) * sin(v)*r,
		       cos(v)*r )

	n.set( index, vector3( du.get(1)*dv.get(2) - du.get(2)*dv.get(1),
			       du.get(2)*dv.get(0) - du.get(0)*dv.get(2),
			       du.get(0)*dv.get(1) - du.get(1)*dv.get(0) ) )

	t.set( index, vector2( i / (N-1), j / (M-1) ) )

    if not (i % 10) :
        sys.stdout.write("\rallocating vertices %d %%" % ((i * 100) / N) )
        sys.stdout.flush()

print

for i in range( 0, N-1 ) :
    for j in range( 0, M-1 ) :
        index = j + i*(M-1)
        
	nv.set( index, 4 )

	fp.set( index*4,   j   + i*M )
	fp.set( index*4+1, j+1 + i*M )
	fp.set( index*4+2, j+1 + (i+1)*M )
	fp.set( index*4+3, j   + (i+1)*M )

	fn.set( index*4,   j   + i*M )
	fn.set( index*4+1, j+1 + i*M )
	fn.set( index*4+2, j+1 + (i+1)*M )
	fn.set( index*4+3, j   + (i+1)*M )

	ft.set( index*4,   j   + i*M )
	ft.set( index*4+1, j+1 + i*M )
	ft.set( index*4+2, j+1 + (i+1)*M )
	ft.set( index*4+3, j   + (i+1)*M )

    if not (i % 10) :
        sys.stdout.write("\rconstructing faces %d %%" % ((i * 100) / N) )
        sys.stdout.flush()

print

s.addObject( s.newObject( "mesh",
			  [ "points",   p,
			    "normals",  n,
			    "textures", t,
			    "faces",    nv.getSize(), nv, fp, fn, ft ] ) )

# destroy arrays
del p
del n
del t
del nv
del fp
del ft
del fn

s.materialEnd()


saver = s.newImager( "tga-saver", [ "file", "mesh1.tga" ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole", [ "eye", vector3( -2, -3, 3 ), "aim", vector3( 0, 0, 0 ) ] )

s.imagerEnd()

s.render( camera, 400, 400 )
