// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfArgType.h                                                                  //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Jacopo Pantaleoni. All rights reserved.           //
// ============================================================================ //

#ifndef _LfArgType_H
#define _LfArgType_H 1

#include <Lightflow/LfBasicTypes.h>
#include <Lightflow/LfError.h>
#include <Lightflow/LfVector.h>
#include <Lightflow/LfArray.h>
#include <Lightflow/LfStdIO.h>


enum LfArgEnum
{
	LfNone_T       = 0,
	LfArgList_T    = 1,
	LfInt_T        = 2,
	LfString_T     = 3,
	LfFloat_T      = 4,
	LfVector2_T    = 5,
	LfVector3_T    = 6,
	LfVector4_T    = 7,
	LfMatrix2_T    = 8,
	LfMatrix3_T    = 9,
	LfMatrix4_T    = 10,
	LfTransform_T  = 11,

	LfIntArray_T       = 12,
	LfFloatArray_T     = 13,
	LfVector2Array_T   = 14,
	LfVector3Array_T   = 15,
	LfVector4Array_T   = 16,
	LfMatrix2Array_T   = 17,
	LfMatrix3Array_T   = 18,
	LfMatrix4Array_T   = 19,
	LfTransformArray_T = 20
};


class LfAPI_CS LfArgType
{
private:
	LfArgEnum    Type;

	LfArgList*   List_V;
	LfInt        Int_V;
	LfString     String_V;
	LfFloat      Float_V;
	LfVector2    Vector2_V;
	LfVector3    Vector3_V;
	LfVector4    Vector4_V;
	LfMatrix2    Matrix2_V;
	LfMatrix3    Matrix3_V;
	LfMatrix4    Matrix4_V;
	LfTransform  Transform_V;

	LfInt       *IntArray_V;
	LfFloat     *FloatArray_V;
	LfVector2   *Vector2Array_V;
	LfVector3   *Vector3Array_V;
	LfVector4   *Vector4Array_V;
	LfMatrix2   *Matrix2Array_V;
	LfMatrix3   *Matrix3Array_V;
	LfMatrix4   *Matrix4Array_V;
	LfTransform *TransformArray_V;

public:
	LfArgType  (void);
	LfArgType  (const LfArgList& list);
	LfArgType  (LfInt value);
	LfArgType  (LfFloat value);
	LfArgType  (LfVector2 value);
	LfArgType  (LfVector3 value);
	LfArgType  (LfVector4 value);
	LfArgType  (LfMatrix2 value);
	LfArgType  (LfMatrix3 value);
	LfArgType  (LfMatrix4 value);
	LfArgType  (LfTransform value);
	LfArgType  (const LfString value);
	LfArgType  (LfInt length, const LfInt       *value);
	LfArgType  (LfInt length, const LfFloat     *value);
	LfArgType  (LfInt length, const LfVector2   *value);
	LfArgType  (LfInt length, const LfVector3   *value);
	LfArgType  (LfInt length, const LfVector4   *value);
	LfArgType  (LfInt length, const LfMatrix2   *value);
	LfArgType  (LfInt length, const LfMatrix3   *value);
	LfArgType  (LfInt length, const LfMatrix4   *value);
	LfArgType  (LfInt length, const LfTransform *value);
	LfArgType  (const LfInstanceArray<LfInt>&       value);
	LfArgType  (const LfInstanceArray<LfFloat>&     value);
	LfArgType  (const LfInstanceArray<LfVector2>&   value);
	LfArgType  (const LfInstanceArray<LfVector3>&   value);
	LfArgType  (const LfInstanceArray<LfVector4>&   value);
	LfArgType  (const LfInstanceArray<LfMatrix2>&   value);
	LfArgType  (const LfInstanceArray<LfMatrix3>&   value);
	LfArgType  (const LfInstanceArray<LfMatrix4>&   value);
	LfArgType  (const LfInstanceArray<LfTransform>& value);
	LfArgType  (const LfArgType& t);
	~LfArgType ();


	LfArgEnum          GetType (void) const { return Type; }

	const LfArgList*   GetList      (void) const;
	LfInt              GetInt       (void) const;
	LfFloat            GetFloat     (void) const;
	LfVector2          GetVector2   (void) const;
	LfVector3          GetVector3   (void) const;
	LfVector4          GetVector4   (void) const;
	LfMatrix2          GetMatrix2   (void) const;
	LfMatrix3          GetMatrix3   (void) const;
	LfMatrix4          GetMatrix4   (void) const;
	LfTransform        GetTransform (void) const;
	const LfString     GetString    (void) const;

	LfInt              GetArraySize      (void) const;
	const LfInt*       GetIntArray       (void) const;
	const LfFloat*     GetFloatArray     (void) const;
	const LfVector2*   GetVector2Array   (void) const;
	const LfVector3*   GetVector3Array   (void) const;
	const LfVector4*   GetVector4Array   (void) const;
	const LfMatrix2*   GetMatrix2Array   (void) const;
	const LfMatrix3*   GetMatrix3Array   (void) const;
	const LfMatrix4*   GetMatrix4Array   (void) const;
	const LfTransform* GetTransformArray (void) const;


	LfArgType& operator = (const LfArgType& t);


	friend LfOAStream& operator<< ( LfOAStream& s, LfArgType  t);
	friend LfIAStream& operator>> ( LfIAStream& s, LfArgType& t);
	friend LfOBStream& operator<< ( LfOBStream& s, LfArgType  t);
	friend LfIBStream& operator>> ( LfIBStream& s, LfArgType& t);

private:
	void Free(void);
};



#endif
