// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfArrayBlock.h                                                               //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Jacopo Pantaleoni. All rights reserved.           //
// ============================================================================ //

#ifndef _LfArrayBlock_H
#define _LfArrayBlock_H 1

#include <WWW/LfBasicTypes.h>
#include <WWW/LfError.h>


template <class TClass> class LfAPI_CS LfInstanceBlock
{
public:
	LfInt   Size;
	TClass* Table;

	LfInstanceBlock<TClass> *Next;

public:
	LfInstanceBlock  (LfInt size = 0) : Size( size ), Table(0), Next(0)
	{
		Table = new TClass [ Size ];
	}
	LfInstanceBlock  (const LfInstanceBlock<TClass>& block ) : Size( block.Size ), Table(0), Next(0)
	{
		Table = new TClass [ Size ];

		for (LfInt i=0; i<Size; i++)
			Table[i] = block.Table[i];
	}

	~LfInstanceBlock (void) { delete [] Table; }

	inline void                     SetNext   (LfInstanceBlock<TClass>* next) { Next = next; }
	inline LfInstanceBlock<TClass>* GetNext   (void) const { return Next; }

	inline LfInt GetSize   (void) const { return Size; }

	inline TClass& operator [] (LfInt i);

	inline const TClass& Get (LfInt i) const;
	inline void          Set (LfInt i, const TClass& n);
};

template <class TClass> class LfAPI_CS LfPointerBlock
{
public:
	LfInt   Size;
	TClass* Table;

	LfPointerBlock<TClass> *Next;

public:
	LfPointerBlock  (LfInt size = 0) : Size( size ), Table(0), Next(0)
	{
		Table = new TClass [ Size ];
	}
	LfPointerBlock  (const LfPointerBlock<TClass>& block ) : Size( block.Size ), Table(0), Next(0)
	{
		Table = new TClass [ Size ];

		for (LfInt i=0; i<Size; i++)
			Table[i] = block.Table[i];
	}

	~LfPointerBlock (void) { delete [] Table; }

	inline void                    SetNext   (LfPointerBlock<TClass>* next) { Next = next; }
	inline LfPointerBlock<TClass>* GetNext   (void) const { return Next; }

	inline LfInt  GetSize   (void) const { return Size; }

	inline TClass operator [] (LfInt i) const;

	inline TClass Get (LfInt i) const;
	inline void   Set (LfInt i, const TClass n);
};

// ---------------------------------------- INSTANTIATION ------------------------------------------ //
LfTEMPLATE_CS( LfInstanceBlock<LfByte> );
LfTEMPLATE_CS( LfInstanceBlock<LfWord> );
LfTEMPLATE_CS( LfInstanceBlock<LfInt> );
LfTEMPLATE_CS( LfInstanceBlock<LfFloat> );
LfTEMPLATE_CS( LfInstanceBlock<LfVector2> );
LfTEMPLATE_CS( LfInstanceBlock<LfVector3> );
LfTEMPLATE_CS( LfInstanceBlock<LfVector4> );
LfTEMPLATE_CS( LfInstanceBlock<LfMatrix2> );
LfTEMPLATE_CS( LfInstanceBlock<LfMatrix3> );
LfTEMPLATE_CS( LfInstanceBlock<LfMatrix4> );
LfTEMPLATE_CS( LfInstanceBlock<LfTransform> );
// ---------------------------------------- IMPLEMENTATION ---------------------------------------- //

template <class TClass> TClass& LfInstanceBlock<TClass>::operator[](LfInt i)
{
	return Table[i];
}
template <class TClass> const TClass& LfInstanceBlock<TClass>::Get(LfInt i) const
{
	return Table[i];
}
template <class TClass> void LfInstanceBlock<TClass>::Set(LfInt i, const TClass& n)
{
	Table[i] = n;
}


template <class TClass> TClass LfPointerBlock<TClass>::operator[](LfInt i) const
{
	return Table[i];
}
template <class TClass> TClass LfPointerBlock<TClass>::Get(LfInt i) const
{
	return Table[i];
}
template <class TClass> void LfPointerBlock<TClass>::Set(LfInt i, TClass n)
{
	Table[i] = n;
}


#endif
