// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfBinarySceneProxy.h                                                         //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Jacopo Pantaleoni. All rights reserved.           //
// ============================================================================ //

#ifndef _LfBinarySceneProxy_H
#define _LfBinarySceneProxy_H 1

#include <WWW/LfSceneProxy.h>
#include <WWW/LfIOStream.h>


class LfAPI_CS LfBinarySceneProxy : public LfSceneProxy
{
private:
	LfOBStream* OStream;

	LfInt CameraNum;
	LfInt ImagerNum;
	LfInt InterfaceNum;
	LfInt InteriorNum;
	LfInt LightNum;
	LfInt MaterialNum;
	LfInt ObjectNum;
	LfInt PatternNum;
	LfInt TextureNum;
	LfInt TrimmerNum;

public:
	LfBinarySceneProxy(LfOBStream* stream);
	virtual ~LfBinarySceneProxy();

// - Entity Constructors and Services ------------------------------------------------------------------------ //
	LfInt  NewCamera        (const LfString type, const LfArgList& arglist);
	LfInt  NewImager        (const LfString type, const LfArgList& arglist);
	LfInt  NewInterface     (const LfString type, const LfArgList& arglist);
	LfInt  NewInterior      (const LfString type, const LfArgList& arglist);
	LfInt  NewLight         (const LfString type, const LfArgList& arglist);
	LfInt  NewMaterial      (const LfString type, const LfArgList& arglist);
	LfInt  NewObject        (const LfString type, const LfArgList& arglist);
	LfInt  NewPattern       (const LfString type, const LfArgList& arglist);
	LfInt  NewTexture       (const LfString name);
	LfInt  NewTrimmer       (const LfString type, const LfArgList& arglist);

	void   ImagerBegin      (LfInt index);
	void   ImagerEnd        (void);

	void   InteriorBegin    (LfInt index);
	void   InteriorEnd      (void);

	void   LightOn          (LfInt index);
	void   LightOff         (LfInt index);
	void   LightBegin       (void);
	void   LightEnd         (void);

	void   MaterialBegin    (LfInt index);
	void   MaterialEnd      (void);

	void   AddObject        (LfInt index);
// ----------------------------------------------------------------------------------------------------------- //

// - Transform Operators ------------------------------------------------------------------------------------- //
	void   TransformBegin   (LfTransform t);
	void   TransformEnd     (void);
// ----------------------------------------------------------------------------------------------------------- //

// - Rendering Operators ------------------------------------------------------------------------------------- //
	void   Radiosity    (void);
	void   Render       (LfInt camera, LfInt w=320, LfInt h=240,
						 LfFloat c1=0, LfFloat r1=0, LfFloat c2=1, LfFloat r2=1);
// ----------------------------------------------------------------------------------------------------------- //
};

enum LfBinarySceneProxyMethods
{
	kNew            = 0,
	kDelete         = 1,

	kNewCamera      = 2,
	kNewImager      = 3,
	kNewInterface   = 4,
	kNewInterior    = 5,
	kNewLight       = 6,
	kNewMaterial    = 7,
	kNewObject      = 8,
	kNewPattern     = 9,
	kNewTexture     = 10,
	kNewTrimmer     = 11,

	kImagerBegin    = 12,
	kImagerEnd      = 13,
	kInteriorBegin  = 14,
	kInteriorEnd    = 15,
	kLightOn        = 16,
	kLightOff       = 17,
	kLightBegin      = 18,
	kLightEnd       = 19,
	kMaterialBegin  = 20,
	kMaterialEnd    = 21,
	kAddObject      = 22,

	kTransformBegin = 23,
	kTransformEnd   = 24,

	kRadiosity      = 25,
	kRender         = 26
};

#endif
