// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfDefs.h                                                                     //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Jacopo Pantaleoni. All rights reserved.           //
// ============================================================================ //

#ifndef _LfDefs_H
#define _LfDefs_H 1

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>


//-------------------------------- OS DEFS -------------------------------------//

#define LfLRI     1
#define LfLRT     1
#define LfDEMO    0

//#define LfWIN     1
//#define LfDLL_CS  1
//#define LfDLL_SS  0
#define LfUNIX    1
#define LfTHREADS 0
#define LfGNU     1
#define LfCONSOLE 1

//--------------------------------- DEBUG --------------------------------------//

#define LfDEBUG   0
#define LfASSERTS 0
#define LfSTATS   0

//------------------------------------------------------------------------------//


#if LfUNIX

#include <unistd.h>

#if LfTHREADS

#define LfPTHREADS 1
#include <pthread.h>

#endif // LfTHREADS

#define LfAPI_CS
#define LfAPI_SS
#define LfTEMPLATE_CS(x)
#define LfTEMPLATE_SS(x)

#endif // LfUNIX

#if LfWIN

#include <io.h>
#include <windows.h>

#if LfTHREADS

#define LfWINTHREADS 1
#include <process.h>

#endif // LfTHREADS

#define LfAPI_CS
#define LfAPI_SS
#define LfTEMPLATE_CS(x)
#define LfTEMPLATE_SS(x)

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the LIGHTFLOW_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// LIGHTFLOW_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.

#if LfDLL_CS

#undef LfAPI_CS

#if LIGHTFLOW_EXPORTS
#define LfAPI_CS __declspec(dllexport)
#else
#define LfAPI_CS __declspec(dllimport)
#endif

#if LIGHTFLOW_TEMPLATE_EXPORTS
#undef  LfTEMPLATE_CS
#define LfTEMPLATE_CS(x) template class LfAPI_CS x
#endif

#endif // LfDLL_CS

#if LfDLL_SS

#undef LfAPI_SS

#if LIGHTFLOW_EXPORTS
#define LfAPI_SS __declspec(dllexport)
#else
#define LfAPI_SS __declspec(dllimport)
#endif

#if LIGHTFLOW_TEMPLATE_EXPORTS
#undef  LfTEMPLATE_SS
#define LfTEMPLATE_SS(x) template class LfAPI_SS x
#endif

#endif // LfDLL_SS

#endif // LfWIN

//-------------------------------- GLOBALS -------------------------------------//

#define LfTrue                 1
#define LfFalse                0

#define LfWarning              0
#define LfFault                1

#define LfUnion                0
#define LfDifference           1
#define LfIntersection         2

#define LfOutside              0
#define LfInside               1
#define LfOnSurface            2

#define LfSurfaceElement       0
#define LfVolumeElement        1

#define LfNotContained         0
#define LfContained            1
#define LfTotallyContained     2

#define LfNone                 0
#define LfLoad                 1
#define LfSave                 2

//-------------------------------- NUMERIC -------------------------------------//

#define LfEpsilon              1e-8  // double: 1e-8, float: 1e-6
#define LfEpsilonSqrt          1e-4
#define LfEpsilonSq            1e-16
#define LfInfinity             1e10
#define LfInfinitySqrt         1e5
#define LfHorizon              1e10
#define LfSurfaceTolerance     1e-6  // double: 1e-6, float: 1e-4
#define LfSurfaceToleranceSqrt 1e-3
#define LfSurfaceToleranceSq   1e-12
#define LfShadowTolerance      1e-3
#define LfShadowToleranceSqrt  4e-2
#define LfShadowToleranceSq    1e-6

//----------------------------- MULTI-THREADING --------------------------------//

#if LfPTHREADS

#define LfMutexFastInitializer      PTHREAD_MUTEX_INITIALIZER
#define LfMutexRecursiveInitializer PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
#define LfMutexFast                 PTHREAD_MUTEX_FAST_NP
#define LfMutexRecursive            PTHREAD_MUTEX_RECURSIVE_NP

#else

#define LfMutexFastInitializer      0
#define LfMutexRecursiveInitializer 0
#define LfMutexFast                 0
#define LfMutexRecursive            0

#endif // LfPTHREADS

//------------------------------------------------------------------------------//


#endif
