// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfStdIO.h                                                                    //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Jacopo Pantaleoni. All rights reserved.           //
// ============================================================================ //

#ifndef _LfStdIO_H
#define _LfStdIO_H 1

#include <WWW/LfIOStream.h>


class LfAPI_CS LfConsoleIAStream : public LfIAStream
{
public:
	LfConsoleIAStream() : LfIAStream() {}
	~LfConsoleIAStream() {}

	void Read (char&           v) { cin >> v; }
	void Read (unsigned char&  v) { cin >> v; }
	void Read (short&          v) { cin >> v; }
	void Read (unsigned short& v) { cin >> v; }
	void Read (int&            v) { cin >> v; }
	void Read (unsigned int&   v) { cin >> v; }
	void Read (long&           v) { cin >> v; }
	void Read (unsigned long&  v) { cin >> v; }
	void Read (float&          v) { cin >> v; }
	void Read (double&         v) { cin >> v; }
	void Read (char*           v) { cin >> v; }
};


class LfAPI_CS LfConsoleOAStream : public LfOAStream
{
public:
	LfConsoleOAStream(void) : LfOAStream() {}
	~LfConsoleOAStream() {}

	void Write (char v)           { cout << v; }
	void Write (unsigned char v)  { cout << v; }
	void Write (short v)          { cout << v; }
	void Write (unsigned short v) { cout << v; }
	void Write (int v)            { cout << v; }
	void Write (unsigned int v)   { cout << v; }
	void Write (long v)           { cout << v; }
	void Write (unsigned long v)  { cout << v; }
	void Write (float v)          { cout << v; }
	void Write (double v)         { cout << v; }
	void Write (const char* v)    { cout << v; }
};


class LfAPI_CS LfFileIOAStream : public LfIAStream, public LfOAStream
{
private:
	FILE* File;

public:
	LfFileIOAStream(const char* filename, const char* mode);
	~LfFileIOAStream();

	LfBool HasSeek(void) const;
	void   Seek(long nbytes);
	long   Tell(void);

	void Read (char&);
	void Read (unsigned char&);
	void Read (short&);
	void Read (unsigned short&);
	void Read (int&);
	void Read (unsigned int&);
	void Read (long&);
	void Read (unsigned long&);
	void Read (float&);
	void Read (double&);
	void Read (char*);

	void Write (char);
	void Write (unsigned char);
	void Write (short);
	void Write (unsigned short);
	void Write (int);
	void Write (unsigned int);
	void Write (long);
	void Write (unsigned long);
	void Write (float);
	void Write (double);
	void Write (const char*);
};


class LfAPI_CS LfFileIOBStream : public LfIBStream, public LfOBStream
{
private:
	FILE* File;

public:
	LfFileIOBStream(const char* filename, const char* mode);
	~LfFileIOBStream();

	LfBool HasSeek(void) const;
	void   Seek(long nbytes);
	long   Tell(void);

	void Read(int nbytes, void* data);
	void Write(int nbytes, const void* data);
};


#endif
