// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfVector2.h                                                                  //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Marco and Jacopo Pantaleoni. All rights reserved. //
// ============================================================================ //

#ifndef _LfVector2_h
#define _LfVector2_h 1

#include <WWW/LfBasicMath.h>
#include <WWW/LfVector3.h>

class LfAPI_CS LfVector2
{
private:
	LfFloat x[2];

public:
inline        LfVector2     ();
inline        LfVector2     (const LfVector2&);
inline        LfVector2     (LfFloat, LfFloat);
inline        LfVector2     (const float  v[]);
inline        LfVector2     (const double v[]);

inline		  LfVector2&	operator  = (const LfVector2&);
inline		  LfVector2&	operator += (const LfVector2&);
inline		  LfVector2&	operator -= (const LfVector2&);
inline		  LfVector2&	operator *= (const LfVector2&);
inline		  LfVector2&	operator *= (LfFloat);
inline		  LfVector2&	operator /= (const LfVector2&);
inline		  LfVector2&	operator /= (LfFloat);
inline		  LfVector2&	Normalize   ();

inline		  LfFloat&      operator [] (int);
inline        LfFloat       Get(int) const;  // This function was introduced by Jacopo Pantaleoni
inline        void          Set(int, LfFloat); // This function was introduced by Jacopo Pantaleoni

inline		  LfFloat       Length      () const;
inline		  LfFloat       SqLength    () const;

friend inline LfAPI_CS int          operator == (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS int          operator != (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS LfVector2	operator  - (const LfVector2&);
friend inline LfAPI_CS LfVector2	operator  + (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS LfVector2	operator  - (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS LfVector2	operator  * (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS LfVector2	operator  * (const LfVector2&,    LfFloat);
friend inline LfAPI_CS LfVector2	operator  * (LfFloat,           const LfVector2&);
friend inline LfAPI_CS LfVector2	operator  * (const LfVector2&,    const LfMatrix2&);
friend inline LfAPI_CS LfVector2	operator  * (const LfMatrix2&,  const LfVector2&);
friend inline LfAPI_CS LfVector2&	Multiply    (const LfVector2&,    const LfMatrix2&,  LfVector2&);
friend inline LfAPI_CS LfVector2&	Multiply    (const LfMatrix2&,  const LfVector2&,    LfVector2&);
friend inline LfAPI_CS LfVector2	operator  / (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS LfVector2	operator  / (const LfVector2&,    LfFloat);
friend inline LfAPI_CS LfVector3    Cross       (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS LfVector3&   Cross       (const LfVector2&,    const LfVector2&,    LfVector3&);
friend inline LfAPI_CS LfVector2	Normalize   (const LfVector2&);
friend inline LfAPI_CS LfFloat		Dot         (const LfVector2&,    const LfVector2&);
friend inline LfAPI_CS LfFloat		Length      (const LfVector2&);
friend inline LfAPI_CS LfFloat		SqLength    (const LfVector2&);
};


//
// I M P L E M E N T A T I O N
//

#ifdef _V
#undef _V
#endif
#define _V(v, i)   (((LfVector2&)v).x[(i)])

#ifdef _M
#undef _M
#endif
#define _M(m, i, j)   (((LfMatrix2&)m).r[(i)][(j)])


//
// LfVector2 inline methods
//

inline LfVector2::LfVector2() { x[0] = x[1] = 0; }

inline LfVector2::LfVector2(const LfVector2& v)
{
	x[0] = v.x[0];
	x[1] = v.x[1];
}

inline LfVector2::LfVector2(LfFloat a, LfFloat b)
{
	x[0] = a;
	x[1] = b;
}

inline LfVector2::LfVector2(const float  v[])
{
	x[0] = (LfFloat)v[0];
	x[1] = (LfFloat)v[1];
}
inline LfVector2::LfVector2(const double v[])
{
	x[0] = (LfFloat)v[0];
	x[1] = (LfFloat)v[1];
}

inline LfVector2& LfVector2::operator = (const LfVector2& v)
{
	x[0] = v.x[0];
	x[1] = v.x[1];
	return *this;
}

inline LfVector2& LfVector2::operator += (const LfVector2& v)
{
	x[0] += v.x[0];
	x[1] += v.x[1];
	return *this;
}

inline LfVector2& LfVector2::operator -= (const LfVector2& v)
{
	x[0] -= v.x[0];
	x[1] -= v.x[1];
	return *this;
}

inline LfVector2& LfVector2::operator *= (const LfVector2& v)
{
	x[0] *= v.x[0];
	x[1] *= v.x[1];
	return *this;
}

inline LfVector2& LfVector2::operator *= (LfFloat m)
{
	x[0] *= m;
	x[1] *= m;
	return *this;
}

inline LfVector2& LfVector2::operator /= (const LfVector2& v)
{
	x[0] /= v.x[0];
	x[1] /= v.x[1];
	return *this;
}

inline LfVector2& LfVector2::operator /= (LfFloat m)
{
	m = 1.0/m;
	x[0] *= m;
	x[1] *= m;
	return *this;
}

inline LfVector2& LfVector2::Normalize()
{
	if (!this->SqLength()) return( *this );
	return (*this /= this->Length());
}

inline LfFloat& LfVector2::operator [] (int i)
{
	return x[i];
}

inline LfFloat LfVector2::Get(int i) const
{
	return x[i];
}
inline void LfVector2::Set(int i , LfFloat c)
{
	x[i] = c;
}

inline LfFloat LfVector2::Length() const
{
	return sqrt(x[0]*x[0]+x[1]*x[1]);
}

inline LfFloat LfVector2::SqLength() const
{
	return (x[0]*x[0]+x[1]*x[1]);
}


//
// LfVector2 inline functions (not members)
//

inline LfAPI_CS int operator == (const LfVector2& v, const LfVector2& w)
{
	return ((v.x[0] == w.x[0]) && (v.x[1] == w.x[1]));
}

inline LfAPI_CS int operator != (const LfVector2& v, const LfVector2& w)
{
	return !(v == w);
}

inline LfAPI_CS LfVector2 operator - (const LfVector2& v)
{
	return (LfVector2(v) *= -1.0);
}

inline LfAPI_CS LfVector2 operator + (const LfVector2& v, const LfVector2& w)
{
	return (LfVector2(v) += w);
}

inline LfAPI_CS LfVector2 operator - (const LfVector2& v, const LfVector2& w)
{
	return (LfVector2(v) -= w);
}

inline LfAPI_CS LfVector2 operator * (const LfVector2& v, const LfVector2& w)
{
	return (LfVector2(v) *= w);
}

inline LfAPI_CS LfVector2 operator * (const LfVector2& v, LfFloat m)
{
	return (LfVector2(v) *= m);
}

inline LfAPI_CS LfVector2 operator * (LfFloat m, const LfVector2& v)
{
	return (LfVector2(v) *= m);
}

inline LfAPI_CS LfVector2 operator / (const LfVector2& v, const LfVector2& w)
{
	return (LfVector2(v) /= w);
}

inline LfAPI_CS LfVector2 operator / (const LfVector2& v, LfFloat m)
{
	return (LfVector2(v) /= m);
}

// SUBOPTIMAL
inline LfAPI_CS LfVector3 Cross(const LfVector2& v, const LfVector2& w)
{
	//
	//    |  i   j   k |
	// det| v0  v1   0 | = k(v0*w1-v1*w0)
	//    | w0  w1   0 |
	//

	return LfVector3(0.0, 0.0, v.x[0]*w.x[1]-v.x[1]*w.x[0]);
}

// OPTIMAL
inline LfAPI_CS LfVector3& Cross(const LfVector2& v, const LfVector2& w, LfVector3& r)
{
	//
	//    |  i   j   k |
	// det| v0  v1   0 | = k(v0*w1-v1*w0)
	//    | w0  w1   0 |
	//

	r.x[0] = r.x[1] = 0.0;
	r.x[2] = v.x[0]*w.x[1] - v.x[1]*w.x[0];
	return r;
}

inline LfAPI_CS LfVector2 Normalize(const LfVector2& v)
{
	if (!v.SqLength()) return( LfVector2(v) );
	return LfVector2(v).Normalize();
}

inline LfAPI_CS LfFloat Dot(const LfVector2& v, const LfVector2& w)
{
	return (v.x[0]*w.x[0] + v.x[1]*w.x[1]);
}

inline LfAPI_CS LfFloat Length(const LfVector2& v) {
	return v.Length();
}

inline LfAPI_CS LfFloat SqLength(const LfVector2& v) {
	return v.SqLength();
}

#undef _V
#undef _M

#endif /* _LfVector2_h */
