// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfVector3.h                                                                  //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Marco and Jacopo Pantaleoni. All rights reserved. //
// ============================================================================ //

#ifndef _LfVector3_h
#define _LfVector3_h 1

#include <WWW/LfBasicMath.h>

class LfAPI_CS LfVector3
{
private:
	LfFloat x[3];

public:
inline        LfVector3       ();
inline        LfVector3       (const LfVector3&);
inline        LfVector3       (LfFloat, LfFloat, LfFloat);
inline        LfVector3       (const float  v[]);
inline        LfVector3       (const double v[]);

inline		  LfVector3&	operator  = (const LfVector3&);
inline		  LfVector3&	operator += (const LfVector3&);
inline		  LfVector3&	operator -= (const LfVector3&);
inline		  LfVector3&	operator *= (const LfVector3&);
inline		  LfVector3&	operator *= (LfFloat);
inline		  LfVector3&	operator /= (const LfVector3&);
inline		  LfVector3&	operator /= (LfFloat);
inline		  LfVector3&	Normalize   ();

inline		  LfFloat&	operator [] (int);
inline        LfFloat   Get(int) const;
inline        void      Set(int, LfFloat);

inline		  LfFloat	Length      () const;
inline		  LfFloat	SqLength    () const;

friend inline LfAPI_CS int		    operator == (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS int		    operator != (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS LfVector3	operator  - (const LfVector3&);
friend inline LfAPI_CS LfVector3	operator  + (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS LfVector3	operator  - (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS LfVector3	operator  * (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS LfVector3	operator  * (const LfVector3&,    LfFloat);
friend inline LfAPI_CS LfVector3	operator  * (LfFloat,             const LfVector3&);
friend inline LfAPI_CS LfVector3	operator  * (const LfVector3&,    const LfMatrix3&);
friend inline LfAPI_CS LfVector3	operator  * (const LfMatrix3&,    const LfVector3&);
friend inline LfAPI_CS LfVector3&	Multiply    (const LfVector3&,    const LfMatrix3&,  LfVector3&);
friend inline LfAPI_CS LfVector3&	Multiply    (const LfMatrix3&,    const LfVector3&,  LfVector3&);
friend inline LfAPI_CS LfVector3	operator  / (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS LfVector3	operator  / (const LfVector3&,    LfFloat);
friend inline LfAPI_CS LfVector3&	Cross	    (const LfVector2&,    const LfVector2&,  LfVector3&);
friend inline LfAPI_CS LfVector3	Cross       (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS LfVector3&	Cross       (const LfVector3&,    const LfVector3&,  LfVector3&);
friend inline LfAPI_CS LfVector3	Normalize   (const LfVector3&);
friend inline LfAPI_CS LfFloat		Dot         (const LfVector3&,    const LfVector3&);
friend inline LfAPI_CS LfFloat		Length      (const LfVector3&);
friend inline LfAPI_CS LfFloat		SqLength    (const LfVector3&);
};


//
// I M P L E M E N T A T I O N
//

#ifdef _V
#undef _V
#endif
#define _V(v, i)   (((LfVector3&)v).x[(i)])

#ifdef _M
#undef _M
#endif
#define _M(m, i, j)   (((LfMatrix3&)m).r[(i)][(j)])


//
// LfVector3 inline methods
//

inline LfVector3::LfVector3() { x[0] = x[1] = x[2] = 0; }

inline LfVector3::LfVector3(const LfVector3& v)
{
	x[0] = v.x[0];
	x[1] = v.x[1];
	x[2] = v.x[2];
}

inline LfVector3::LfVector3(LfFloat a, LfFloat b, LfFloat c)
{
	x[0] = a;
	x[1] = b;
	x[2] = c;
}

inline LfVector3::LfVector3(const float v[])
{
	x[0] = (LfFloat)v[0];
	x[1] = (LfFloat)v[1];
	x[2] = (LfFloat)v[2];
}
inline LfVector3::LfVector3(const double v[])
{
	x[0] = (LfFloat)v[0];
	x[1] = (LfFloat)v[1];
	x[2] = (LfFloat)v[2];
}

inline LfVector3& LfVector3::operator = (const LfVector3& v)
{
	x[0] = v.x[0];
	x[1] = v.x[1];
	x[2] = v.x[2];
	return *this;
}

inline LfVector3& LfVector3::operator += (const LfVector3& v)
{
	x[0] += v.x[0];
	x[1] += v.x[1];
	x[2] += v.x[2];
	return *this;
}

inline LfVector3& LfVector3::operator -= (const LfVector3& v)
{
	x[0] -= v.x[0];
	x[1] -= v.x[1];
	x[2] -= v.x[2];
	return *this;
}

inline LfVector3& LfVector3::operator *= (const LfVector3& v)
{
	x[0] *= v.x[0];
	x[1] *= v.x[1];
	x[2] *= v.x[2];
	return *this;
}

inline LfVector3& LfVector3::operator *= (LfFloat m)
{
	x[0] *= m;
	x[1] *= m;
	x[2] *= m;
	return *this;
}

inline LfVector3& LfVector3::operator /= (const LfVector3& v)
{
	x[0] /= v.x[0];
	x[1] /= v.x[1];
	x[2] /= v.x[2];
	return *this;
}

inline LfVector3& LfVector3::operator /= (LfFloat m) // Optimized by J.Pantaleoni
{
	m = 1.0/m;
	x[0] *= m;
	x[1] *= m;
	x[2] *= m;
	return *this;
}

inline LfVector3& LfVector3::Normalize() // Optimized by J.Pantaleoni
{
	LfFloat invlen = Length();
	if (!invlen) return(*this);

	invlen = 1.0/invlen;
	x[0] *= invlen;
	x[1] *= invlen;
	x[2] *= invlen;

	return (*this);
}

inline LfFloat& LfVector3::operator [] (int i)
{
	return x[i];
}

inline LfFloat LfVector3::Get(int i) const
{
	return x[i];
}
inline void LfVector3::Set(int i , LfFloat c)
{
	x[i] = c;
}

inline LfFloat LfVector3::Length() const
{
	return sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2]);
}

inline LfFloat LfVector3::SqLength() const
{
	return (x[0]*x[0] + x[1]*x[1] + x[2]*x[2]);
}


//
// LfVector3 inline functions (not members)
//

inline LfAPI_CS int operator == (const LfVector3& v, const LfVector3& w)
{
	return ((v.x[0] == w.x[0]) && (v.x[1] == w.x[1]) && (v.x[2] == w.x[2]));
}

inline LfAPI_CS int operator != (const LfVector3& v, const LfVector3& w)
{
	return !(v == w);
}

inline LfAPI_CS LfVector3 operator - (const LfVector3& v)
{
	return (LfVector3(v) *= -1.0);
}

inline LfAPI_CS LfVector3 operator + (const LfVector3& v, const LfVector3& w)
{
	return (LfVector3(v) += w);
}

inline LfAPI_CS LfVector3 operator - (const LfVector3& v, const LfVector3& w)
{
	return (LfVector3(v) -= w);
}

inline LfAPI_CS LfVector3 operator * (const LfVector3& v, const LfVector3& w)
{
	return (LfVector3(v) *= w);
}

inline LfAPI_CS LfVector3 operator * (const LfVector3& v, LfFloat m)
{
	return (LfVector3(v) *= m);
}

inline LfAPI_CS LfVector3 operator * (LfFloat m, const LfVector3& v)
{
	return (LfVector3(v) *= m);
}

inline LfAPI_CS LfVector3 operator / (const LfVector3& v, const LfVector3& w)
{
	return (LfVector3(v) /= w);
}

inline LfAPI_CS LfVector3 operator / (const LfVector3& v, LfFloat m)
{
	return (LfVector3(v) /= m);
}

inline LfAPI_CS LfVector3 Cross(const LfVector3& v, const LfVector3& w)
{
        //
        //    |  i   j   k |
        // det| v0  v1  v2 | = i(v1*w2-v2*w1)-j(v0*w2-v2*w0)+k(v0*w1-v1*w0)
        //    | w0  w1  w2 |
        //
	LfVector3 r;

	return Cross(v, w, r);
}

inline LfAPI_CS LfVector3& Cross(const LfVector3& v, const LfVector3& w, LfVector3& r)
{
	//
	//    |  i   j   k |
	// det| v0  v1  v2 | = i(v1*w2-v2*w1)-j(v0*w2-v2*w0)+k(v0*w1-v1*w0)
	//    | w0  w1  w2 |
	//

	r.x[0] = v.x[1]*w.x[2]-v.x[2]*w.x[1];
	r.x[1] = v.x[2]*w.x[0]-v.x[0]*w.x[2];
	r.x[2] = v.x[0]*w.x[1]-v.x[1]*w.x[0];

	return r;
}

inline LfAPI_CS LfVector3 Normalize(const LfVector3& v)
{
	if (!v.SqLength()) return( LfVector3(v) );
	return LfVector3(v).Normalize();
}

inline LfAPI_CS LfFloat Dot(const LfVector3& v, const LfVector3& w)
{
	return (v.x[0]*w.x[0] + v.x[1]*w.x[1] + v.x[2]*w.x[2]);
}

inline LfAPI_CS LfFloat Length(const LfVector3& v)
{
	return v.Length();
}

inline LfAPI_CS LfFloat SqLength(const LfVector3& v)
{
	return v.SqLength();
}

#undef _V
#undef _M

#endif /* _LfVector3_h */
