// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfVector4.h                                                                  //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Marco and Jacopo Pantaleoni. All rights reserved. //
// ============================================================================ //

#ifndef _LfVector4_h
#define _LfVector4_h 1

#include <WWW/LfBasicMath.h>

class LfAPI_CS LfVector4
{
private:
	LfFloat x[4];

public:
inline        LfVector4     ();
inline        LfVector4     (const LfVector4&);
inline        LfVector4     (LfFloat, LfFloat, LfFloat, LfFloat);
inline        LfVector4     (const float  v[]);
inline        LfVector4     (const double v[]);

inline		  LfVector4&	operator  = (const LfVector4&);
inline		  LfVector4&	operator += (const LfVector4&);
inline		  LfVector4&	operator -= (const LfVector4&);
inline		  LfVector4&	operator *= (const LfVector4&);
inline		  LfVector4&	operator *= (LfFloat);
inline		  LfVector4&	operator /= (const LfVector4&);
inline		  LfVector4&	operator /= (LfFloat);
inline		  LfVector4&	Normalize   ();

inline		  LfFloat&	operator [] (int);
inline        LfFloat   Get(int) const;  // This function was introduced by Jacopo Pantaleoni
inline        void      Set(int, LfFloat); // This function was introduced by Jacopo Pantaleoni

inline		  LfFloat	Length      () const;
inline		  LfFloat	SqLength    () const;

friend inline LfAPI_CS int		    operator == (const LfVector4&,    const LfVector4&);
friend inline LfAPI_CS int		    operator != (const LfVector4&,    const LfVector4&);
friend inline LfAPI_CS LfVector4	operator  - (const LfVector4&);
friend inline LfAPI_CS LfVector4	operator  + (const LfVector4&,    const LfVector4&);
friend inline LfAPI_CS LfVector4	operator  - (const LfVector4&,    const LfVector4&);
friend inline LfAPI_CS LfVector4	operator  * (const LfVector4&,    const LfVector4&);
friend inline LfAPI_CS LfVector4	operator  * (const LfVector4&,    LfFloat);
friend inline LfAPI_CS LfVector4	operator  * (LfFloat,           const LfVector4&);
friend inline LfAPI_CS LfVector4	operator  * (const LfVector4&,    const LfMatrix4&);
friend inline LfAPI_CS LfVector4	operator  * (const LfMatrix4&,  const LfVector4&);
friend inline LfAPI_CS LfVector4&	Multiply    (const LfVector4&,    const LfMatrix4&,  LfVector4&);
friend inline LfAPI_CS LfVector4&	Multiply    (const LfMatrix4&,  const LfVector4&,    LfVector4&);
friend inline LfAPI_CS LfVector4	operator  / (const LfVector4&,    const LfVector4&);
friend inline LfAPI_CS LfVector4	operator  / (const LfVector4&,    LfFloat);
friend inline LfAPI_CS LfVector4	Normalize   (const LfVector4&);
friend inline LfAPI_CS LfFloat		Dot         (const LfVector4&,    const LfVector4&);
friend inline LfAPI_CS LfFloat		Length      (const LfVector4&);
friend inline LfAPI_CS LfFloat		SqLength    (const LfVector4&);
};


//
// I M P L E M E N T A T I O N
//

#ifdef _V
#undef _V
#endif
#define _V(v, i)   (((LfVector4&)v).x[(i)])

#ifdef _M
#undef _M
#endif
#define _M(m, i, j)   (((LfMatrix4&)m).r[(i)][(j)])


//
// LfVector4 inline methods
//

inline LfVector4::LfVector4() { x[0] = x[1] = x[2] = x[3] = 0; }

inline LfVector4::LfVector4(const LfVector4& v)
{
	x[0] = v.x[0];
	x[1] = v.x[1];
	x[2] = v.x[2];
	x[3] = v.x[3];
}

inline LfVector4::LfVector4(LfFloat a, LfFloat b, LfFloat c, LfFloat d)
{
	x[0] = a;
	x[1] = b;
	x[2] = c;
	x[3] = d;
}
inline LfVector4::LfVector4(const float  v[])
{
	x[0] = (LfFloat)v[0];
	x[1] = (LfFloat)v[1];
	x[2] = (LfFloat)v[2];
	x[3] = (LfFloat)v[3];
}
inline LfVector4::LfVector4(const double v[])
{
	x[0] = (LfFloat)v[0];
	x[1] = (LfFloat)v[1];
	x[2] = (LfFloat)v[2];
	x[3] = (LfFloat)v[3];
}

inline LfVector4& LfVector4::operator = (const LfVector4& v)
{
	x[0] = v.x[0];
	x[1] = v.x[1];
	x[2] = v.x[2];
	x[3] = v.x[3];
	return *this;
}

inline LfVector4& LfVector4::operator += (const LfVector4& v)
{
	x[0] += v.x[0];
	x[1] += v.x[1];
	x[2] += v.x[2];
	x[3] += v.x[3];
	return *this;
}

inline LfVector4& LfVector4::operator -= (const LfVector4& v)
{
	x[0] -= v.x[0];
	x[1] -= v.x[1];
	x[2] -= v.x[2];
	x[3] -= v.x[3];
	return *this;
}

inline LfVector4& LfVector4::operator *= (const LfVector4& v)
{
	x[0] *= v.x[0];
	x[1] *= v.x[1];
	x[2] *= v.x[2];
	x[3] *= v.x[3];
	return *this;
}

inline LfVector4& LfVector4::operator *= (LfFloat m)
{
	x[0] *= m;
	x[1] *= m;
	x[2] *= m;
	x[3] *= m;
	return *this;
}

inline LfVector4& LfVector4::operator /= (const LfVector4& v)
{
	x[0] /= v.x[0];
	x[1] /= v.x[1];
	x[2] /= v.x[2];
	x[3] /= v.x[3];
	return *this;
}

inline LfVector4& LfVector4::operator /= (LfFloat m)
{
	m = 1.0/m;
	x[0] *= m;
	x[1] *= m;
	x[2] *= m;
	x[3] *= m;
	return *this;
}

inline LfVector4& LfVector4::Normalize()
{
	if (!this->SqLength()) return( *this );
	return (*this /= this->Length());
}

inline LfFloat& LfVector4::operator [] (int i)
{
	return x[i];
}

inline LfFloat LfVector4::Get(int i) const
{
	return x[i];
}
inline void LfVector4::Set(int i , LfFloat c)
{
	x[i] = c;
}

inline LfFloat LfVector4::Length() const
{
	return sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2] + x[3]*x[3]);
}

inline LfFloat LfVector4::SqLength() const
{
	return (x[0]*x[0] + x[1]*x[1] + x[2]*x[2] + x[3]*x[3]);
}


//
// LfVector4 inline functions (not members)
//

inline LfAPI_CS int operator == (const LfVector4& v, const LfVector4& w)
{
	return ((v.x[0] == w.x[0]) && (v.x[1] == w.x[1]) && (v.x[2] == w.x[2]) && (v.x[3] == w.x[3]));
}

inline LfAPI_CS int operator != (const LfVector4& v, const LfVector4& w)
{
	return !(v == w);
}

inline LfAPI_CS LfVector4 operator - (const LfVector4& v)
{
	return (LfVector4(v) *= -1.0);
}

inline LfAPI_CS LfVector4 operator + (const LfVector4& v, const LfVector4& w)
{
	return (LfVector4(v) += w);
}

inline LfAPI_CS LfVector4 operator - (const LfVector4& v, const LfVector4& w)
{
	return (LfVector4(v) -= w);
}

inline LfAPI_CS LfVector4 operator * (const LfVector4& v, const LfVector4& w)
{
	return (LfVector4(v) *= w);
}

inline LfAPI_CS LfVector4 operator * (const LfVector4& v, LfFloat m)
{
	return (LfVector4(v) *= m);
}

inline LfAPI_CS LfVector4 operator * (LfFloat m, const LfVector4& v)
{
	return (LfVector4(v) *= m);
}

inline LfAPI_CS LfVector4 operator / (const LfVector4& v, const LfVector4& w)
{
	return (LfVector4(v) /= w);
}

inline LfAPI_CS LfVector4 operator / (const LfVector4& v, LfFloat m)
{
	return (LfVector4(v) /= m);
}

inline LfAPI_CS LfVector4 Normalize(const LfVector4& v)
{
	if (!v.SqLength()) return( LfVector4(v) );
	return LfVector4(v).Normalize();
}

inline LfAPI_CS LfFloat Dot(const LfVector4& v, const LfVector4& w)
{
	return (v.x[0]*w.x[0] + v.x[1]*w.x[1] + v.x[2]*w.x[2] + v.x[3]*w.x[3]);
}

inline LfAPI_CS LfFloat Length(const LfVector4& v)
{
	return v.Length();
}

inline LfAPI_CS LfFloat SqLength(const LfVector4& v)
{
	return v.SqLength();
}

#undef _V
#undef _M

#endif /* _LfVector4_h */
